package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PeriodVerificationPlanDTO;
import digital.laboratory.platform.reagent.entity.CheckSchedule;
import digital.laboratory.platform.reagent.entity.PeriodVerificationPlan;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.service.CheckScheduleService;
import digital.laboratory.platform.reagent.service.InstructionBookService;
import digital.laboratory.platform.reagent.service.PeriodVerificationPlanService;
import digital.laboratory.platform.reagent.vo.AcceptanceRecordFormVO;
import digital.laboratory.platform.reagent.vo.CheckScheduleVO;
import org.joda.time.DateTime;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.util.Date;
import java.util.List;

/**
 * @author Zhang Xiaolong created at  2023-04-10
 * @describe 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/check_schedule")
@Api(value = "check_schedule", tags = "期间核查计划制定管理")
public class CheckScheduleController {

    private final CheckScheduleService checkScheduleService;

    private final OssFile ossFile;

    private final PeriodVerificationPlanService periodVerificationPlanService;


    /**
     * 通过id查询
     *
     * @param checkScheduleId id
     * @return R
     */
    @ApiOperation(value = "通过id查询新增标准物质期间核查计划和确认表", notes = "通过id查询新增标准物质期间核查计划和确认表")
    @GetMapping()
	@PreAuthorize("@pms.hasPermission('reagent_check_schedule_get')" )
    public R<CheckSchedule> getById(String checkScheduleId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CheckSchedule checkSchedule = checkScheduleService.getCheckScheduleVO(checkScheduleId);
        return R.ok(checkSchedule);
        //return R.ok(checkScheduleService.getById(managerId));
    }

    /**
     * 通过id删除
     *
     * @param  id
     * @return R
     */
    @ApiOperation(value = "通过id删除期间核查计划明细", notes = "通过id删除期间核查计划明细")
    @DeleteMapping()
//	@PreAuthorize("@pms.hasPermission('reagent_check_schedule_del')" )
    public R<String > delById(String id, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PeriodVerificationPlan byId = periodVerificationPlanService.getById(id);
        CheckScheduleVO checkScheduleVO = checkScheduleService.getCheckScheduleVO(byId.getCheckScheduleId());
        if (checkScheduleVO.getPeriodVerificationPlanVOS().size()==1) {
            periodVerificationPlanService.removeById(byId);
            checkScheduleService.removeById(checkScheduleVO.getId());
            return R.ok("删除成功");
        }
        else {
            return R.failed("删除失败");
        }
    }

    /**
     * 分页查询
     *
     * @param page          分页对象
     * @param page
     * @return
     */
    @ApiOperation(value = "分页查询新增标准物质期间核查计划和确认表", notes = "分页查询新增标准物质期间核查计划和确认表")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_check_schedule_page')" )
    public R<IPage<CheckScheduleVO>> getCheckSchedulePage(Page<CheckSchedule> page, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") Date startTime, @DateTimeFormat(pattern = "yyyy-MM-dd") @JsonFormat(pattern = "yyyy-MM-dd") Date endTime, String status, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<CheckScheduleVO> checkScheduleVOPage = checkScheduleService.getCheckScheduleVOPage(page, Wrappers.<CheckSchedule>query()
                .ge(startTime!=null,"create_time", startTime)
                .le(endTime!=null,"create_time", endTime)
                .eq(StrUtil.isNotBlank(status), "status", status)
                .orderByDesc("create_time"));

        return R.ok(checkScheduleVOPage);
    }

    /**
     * 新增
     *
     * @param periodVerificationPlanDTOS
     * @return R
     */
    @ApiOperation(value = "新增标准物质期间核查计划和确认表", notes = "新增标准物质期间核查计划和确认表")
    @SysLog("新增")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_check_schedule_add')" )
    public R<CheckScheduleVO> postAddObject(@RequestBody List<PeriodVerificationPlanDTO> periodVerificationPlanDTOS, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CheckScheduleVO checkSchedule = checkScheduleService.addPlan(periodVerificationPlanDTOS, dlpUser);

        if (checkSchedule != null) {
            return R.ok(checkSchedule, "计划创建成功");
        } else {
            return R.failed("计划创建失败");
        }
    }

    /**
     * 修改
     *
     * @param periodVerificationPlanDTOS
     * @return R
     */
    @ApiOperation(value = "修改标准物质期间核查计划和确认表", notes = "修改标准物质期间核查计划和确认表")
    @SysLog("修改")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_check_schedule_edit')" )
    public R<CheckSchedule> putUpdateById(@RequestBody List<PeriodVerificationPlanDTO> periodVerificationPlanDTOS, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CheckSchedule checkSchedule = checkScheduleService.editPlan(periodVerificationPlanDTOS);

        if (checkSchedule != null) {
            return R.ok(checkSchedule, "保存成功");
        } else return R.failed("保存失败");
    }

    /**
     * 提交计划
     *
     * @param periodVerificationPlanDTOS
     * @return R
     */
    @ApiOperation(value = "提交计划标准物质期间核查计划和确认表", notes = "提交计划标准物质期间核查计划和确认表")
    @SysLog("提交计划")
    @PostMapping("/commit")
    @PreAuthorize("@pms.hasPermission('reagent_check_schedule_commit')" )
    public R<CheckSchedule> commitById(@RequestBody List<PeriodVerificationPlanDTO> periodVerificationPlanDTOS, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CheckSchedule checkSchedule = checkScheduleService.commitPlan(periodVerificationPlanDTOS, dlpUser);

        if (checkSchedule != null) {
            return R.ok(checkSchedule, "提交成功");
        } else return R.failed("提交失败");
    }

    /**
     * 审核
     *
     * @param auditAndApproveDTO
     * @return R
     */
    @ApiOperation(value = "审核标准物质期间核查计划和确认表", notes = "审核标准物质期间核查计划和确认表")
    @SysLog("修改")
    @PutMapping("/audit")
    @PreAuthorize("@pms.hasPermission('reagent_check_schedule_audit')")
    public R<CheckSchedule> auditPlan(@RequestBody AuditAndApproveDTO auditAndApproveDTO, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CheckSchedule checkSchedule = checkScheduleService.auditPlan(auditAndApproveDTO, dlpUser);

        if (checkSchedule != null) {
            return R.ok(checkSchedule, "审核成功");
        } else return R.failed("审核失败");
    }

    /**-------验收记录打印*/
    /**
     * 验收记录打印
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "期间核查计划打印", notes = "期间核查计划打印")
    @SysLog("期间核查计划打印")
    @PostMapping("/print")
	@PreAuthorize("@pms.hasPermission('reagent_check_schedule_print')")
    public void bizGetPDFInventory(String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CheckScheduleVO checkScheduleVO = checkScheduleService.getCheckScheduleVO(id);

        String applyFileName = "标准物质期间核查计划和确认表" + id;
        ;


        String pdfFilePath = "document" + "/" + "checkSchedule" + "/" + id + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            checkScheduleService.checkScheduleTablePDF(checkScheduleVO, theHttpServletRequest, httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }


}
