package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.OutgoingContentsDTO;
import digital.laboratory.platform.reagent.entity.DeliveryRegistrationForm;
import digital.laboratory.platform.reagent.entity.OutgoingContents;
import digital.laboratory.platform.reagent.service.DeliveryRegistrationFormService;
import digital.laboratory.platform.reagent.service.OutgoingContentsService;
import digital.laboratory.platform.reagent.vo.DeliveryRegistrationFormVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumablesVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * (试剂耗材出库登记表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材出库登记表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/delivery_registration_form" )
@Api(value = "delivery_registration_form", tags = "(试剂耗材出库登记表)管理")
public class DeliveryRegistrationFormController {

    private final  DeliveryRegistrationFormService deliveryRegistrationFormService;

	private final OutgoingContentsService outgoingContentsService;

	/**
	 * 通过id查询(试剂耗材出库登记表)
	 * @param deliveryRegistrationFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过ID查询", notes = "通过ID查询")
	@GetMapping( )
	@PreAuthorize("@pms.hasPermission('reagent_delivery_registration_form_get')" )
	public R<DeliveryRegistrationFormVO> getById(String deliveryRegistrationFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        DeliveryRegistrationFormVO deliveryRegistrationForm = deliveryRegistrationFormService.getDeliveryRegistrationFormVOById(deliveryRegistrationFormId);
		return R.ok(deliveryRegistrationForm);
		//return R.ok(deliveryRegistrationFormService.getById(deliveryRegistrationFormId));
	}

	/**
	 * 通过领取码(试剂耗材出库登记表)
	 * @param claimCode id
	 * @return R
	 */
	@ApiOperation(value = "通过领取码查询", notes = "通过领取码查询")
	@GetMapping("/code")
	@PreAuthorize("@pms.hasPermission('reagent_delivery_registration_form_get')" )
	public R<DeliveryRegistrationFormVO> getDeliveryRegistrationFormVOByCode(String claimCode, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		DeliveryRegistrationFormVO deliveryRegistrationForm = deliveryRegistrationFormService.getDeliveryRegistrationFormVOByCode(claimCode);
		return R.ok(deliveryRegistrationForm);
		//return R.ok(deliveryRegistrationFormService.getById(deliveryRegistrationFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param deliveryRegistrationForm (试剂耗材出库登记表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_delivery_registration_form_page')" )
    public R<IPage<DeliveryRegistrationFormVO>> getDeliveryRegistrationFormPage(Page<DeliveryRegistrationForm> page, DeliveryRegistrationForm deliveryRegistrationForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<DeliveryRegistrationFormVO> deliveryRegistrationFormSList = deliveryRegistrationFormService.getDeliveryRegistrationFormVOPage(page, Wrappers.<DeliveryRegistrationForm>query()
						.eq("status",0)
				.orderByDesc("create_time")
		);

		List<DeliveryRegistrationFormVO> records = deliveryRegistrationFormSList.getRecords();

		for (DeliveryRegistrationFormVO record : records) {

			LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();

			outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId,record.getId());

			record.setTypeOfDelivery(outgoingContentsService.list(outgoingContentsLambdaQueryWrapper).size());
		}
		return R.ok(deliveryRegistrationFormSList);
//		return R.ok(deliveryRegistrationFormService.page(page, Wrappers.query(deliveryRegistrationForm)));
    }


    /**
     * 提交(试剂耗材出库登记表)
     * @param outgoingContentsDTOS (试剂耗材出库登记表)
     * @return R
     */
    @ApiOperation(value = "提交(试剂耗材出库登记表)", notes = "提交(试剂耗材出库登记表)")
    @SysLog("提交(试剂耗材出库登记表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_delivery_registration_form_commit')" )
    public R<DeliveryRegistrationForm> postAddObject(@RequestBody List<OutgoingContentsDTO> outgoingContentsDTOS, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		DeliveryRegistrationForm deliveryRegistrationForm = deliveryRegistrationFormService.commitForm(outgoingContentsDTOS, dlpUser);

		if (deliveryRegistrationForm!=null){
			return R.ok(deliveryRegistrationForm,"提交成功");
		}else return R.failed("提交失败");
	}


}
